// Copyright (c) 2020 RISC Software GmbH
//
// This file was generated by CPACSGen from CPACS XML Schema (c) German Aerospace Center (DLR/SC).
// Do not edit, all changes are lost when files are re-generated.
//
// Licensed under the Apache License, Version 2.0 (the "License")
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <cassert>
#include "CCPACSEngines.h"
#include "CPACSEngine.h"
#include "CTiglError.h"
#include "CTiglLogging.h"
#include "CTiglUIDManager.h"
#include "TixiHelper.h"

namespace tigl
{
namespace generated
{
    CPACSEngine::CPACSEngine(CCPACSEngines* parent, CTiglUIDManager* uidMgr)
        : m_uidMgr(uidMgr)
    {
        //assert(parent != NULL);
        m_parent = parent;
    }

    CPACSEngine::~CPACSEngine()
    {
        if (m_uidMgr) m_uidMgr->TryUnregisterObject(m_uID);
    }

    const CCPACSEngines* CPACSEngine::GetParent() const
    {
        return m_parent;
    }

    CCPACSEngines* CPACSEngine::GetParent()
    {
        return m_parent;
    }

    const CTiglUIDObject* CPACSEngine::GetNextUIDParent() const
    {
        if (m_parent) {
            return m_parent->GetNextUIDParent();
        }
        return nullptr;
    }

    CTiglUIDObject* CPACSEngine::GetNextUIDParent()
    {
        if (m_parent) {
            return m_parent->GetNextUIDParent();
        }
        return nullptr;
    }

    CTiglUIDManager& CPACSEngine::GetUIDManager()
    {
        return *m_uidMgr;
    }

    const CTiglUIDManager& CPACSEngine::GetUIDManager() const
    {
        return *m_uidMgr;
    }

    void CPACSEngine::ReadCPACS(const TixiDocumentHandle& tixiHandle, const std::string& xpath)
    {
        // read attribute uID
        if (tixi::TixiCheckAttribute(tixiHandle, xpath, "uID")) {
            m_uID = tixi::TixiGetAttribute<std::string>(tixiHandle, xpath, "uID");
            if (m_uID.empty()) {
                LOG(WARNING) << "Required attribute uID is empty at xpath " << xpath;
            }
        }
        else {
            LOG(ERROR) << "Required attribute uID is missing at xpath " << xpath;
        }

        // read element name
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/name")) {
            m_name = tixi::TixiGetElement<std::string>(tixiHandle, xpath + "/name");
            if (m_name.empty()) {
                LOG(WARNING) << "Required element name is empty at xpath " << xpath;
            }
        }
        else {
            LOG(ERROR) << "Required element name is missing at xpath " << xpath;
        }

        // read element description
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/description")) {
            m_description = tixi::TixiGetElement<std::string>(tixiHandle, xpath + "/description");
            if (m_description->empty()) {
                LOG(WARNING) << "Optional element description is present but empty at xpath " << xpath;
            }
        }

        // read element thrust00Scaling
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/thrust00Scaling")) {
            m_thrust00Scaling = tixi::TixiGetElement<double>(tixiHandle, xpath + "/thrust00Scaling");
        }

        // read element nacelle
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/nacelle")) {
            m_nacelle = boost::in_place(this, m_uidMgr);
            try {
                m_nacelle->ReadCPACS(tixiHandle, xpath + "/nacelle");
            } catch(const std::exception& e) {
                LOG(ERROR) << "Failed to read nacelle at xpath " << xpath << ": " << e.what();
                m_nacelle = boost::none;
            }
        }

        if (m_uidMgr && !m_uID.empty()) m_uidMgr->RegisterObject(m_uID, *this);
    }

    void CPACSEngine::WriteCPACS(const TixiDocumentHandle& tixiHandle, const std::string& xpath) const
    {
        // write attribute uID
        tixi::TixiSaveAttribute(tixiHandle, xpath, "uID", m_uID);

        // write element name
        tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/name");
        tixi::TixiSaveElement(tixiHandle, xpath + "/name", m_name);

        // write element description
        if (m_description) {
            tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/description");
            tixi::TixiSaveElement(tixiHandle, xpath + "/description", *m_description);
        }
        else {
            if (tixi::TixiCheckElement(tixiHandle, xpath + "/description")) {
                tixi::TixiRemoveElement(tixiHandle, xpath + "/description");
            }
        }

        // write element thrust00Scaling
        if (m_thrust00Scaling) {
            tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/thrust00Scaling");
            tixi::TixiSaveElement(tixiHandle, xpath + "/thrust00Scaling", *m_thrust00Scaling);
        }
        else {
            if (tixi::TixiCheckElement(tixiHandle, xpath + "/thrust00Scaling")) {
                tixi::TixiRemoveElement(tixiHandle, xpath + "/thrust00Scaling");
            }
        }

        // write element nacelle
        if (m_nacelle) {
            tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/nacelle");
            m_nacelle->WriteCPACS(tixiHandle, xpath + "/nacelle");
        }
        else {
            if (tixi::TixiCheckElement(tixiHandle, xpath + "/nacelle")) {
                tixi::TixiRemoveElement(tixiHandle, xpath + "/nacelle");
            }
        }

    }

    const std::string& CPACSEngine::GetUID() const
    {
        return m_uID;
    }

    void CPACSEngine::SetUID(const std::string& value)
    {
        if (m_uidMgr && value != m_uID) {
            if (m_uID.empty()) {
                m_uidMgr->RegisterObject(value, *this);
            }
            else {
                m_uidMgr->UpdateObjectUID(m_uID, value);
            }
        }
        m_uID = value;
    }

    const std::string& CPACSEngine::GetName() const
    {
        return m_name;
    }

    void CPACSEngine::SetName(const std::string& value)
    {
        m_name = value;
    }

    const boost::optional<std::string>& CPACSEngine::GetDescription() const
    {
        return m_description;
    }

    void CPACSEngine::SetDescription(const boost::optional<std::string>& value)
    {
        m_description = value;
    }

    const boost::optional<double>& CPACSEngine::GetThrust00Scaling() const
    {
        return m_thrust00Scaling;
    }

    void CPACSEngine::SetThrust00Scaling(const boost::optional<double>& value)
    {
        m_thrust00Scaling = value;
    }

    const boost::optional<CPACSEngineNacelle>& CPACSEngine::GetNacelle() const
    {
        return m_nacelle;
    }

    boost::optional<CPACSEngineNacelle>& CPACSEngine::GetNacelle()
    {
        return m_nacelle;
    }

    CPACSEngineNacelle& CPACSEngine::GetNacelle(CreateIfNotExistsTag)
    {
        if (!m_nacelle)
            m_nacelle = boost::in_place(this, m_uidMgr);
        return *m_nacelle;
    }

    void CPACSEngine::RemoveNacelle()
    {
        m_nacelle = boost::none;
    }

} // namespace generated
} // namespace tigl
